/*
    Xspider, an X11 spider friend
    Copyright (C) 2022  alicia@ion.nu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <math.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

GdkPixbuf* rotatescale(GdkPixbuf* src, double angle, int size)
{
  GdkPixbuf* dest=gdk_pixbuf_new(GDK_COLORSPACE_RGB, 1, 8, size, size);
  guchar* srcp=gdk_pixbuf_get_pixels(src);
  guchar* destp=gdk_pixbuf_get_pixels(dest);
  int destrowstride=gdk_pixbuf_get_rowstride(dest);
  int destpstride=gdk_pixbuf_get_n_channels(dest);
  int srcrowstride=gdk_pixbuf_get_rowstride(src);
  int srcpstride=gdk_pixbuf_get_n_channels(src);
  int srcw=gdk_pixbuf_get_width(src);
  int srch=gdk_pixbuf_get_height(src);
  int srcsize=MAX(srcw,srch);
  int psize=srcsize/size;
  int x,y;
  for(y=0; y<size; y++)
  {
    for(x=0; x<size; x++)
    {
      unsigned int samples[4]={0};
      int i,x2,y2;
      int x2base=cos(angle)*(x*psize-srcsize/2)+sin(angle)*(y*psize-srcsize/2)+srcw/2;
      int y2base=cos(angle)*(y*psize-srcsize/2)-sin(angle)*(x*psize-srcsize/2)+srch/2;
      for(y2=y2base; y2<y2base+psize; y2++)
      {
        if(y2<0 || y2>=srch){continue;}
        for(x2=x2base; x2<x2base+psize; x2++)
        {
          if(x2<0 || x2>=srcw){continue;}
          for(i=0; i<4; i++)
          {
            samples[i]+=srcp[y2*srcrowstride+x2*srcpstride+i];
          }
        }
      }
      for(i=0; i<4; i++)
      {
        destp[y*destrowstride+x*destpstride+i]=samples[i]/psize/psize;
      }
    }
  }
  return dest;
}
